<?php
/**
 * @file
 * Contains \Drupal\guide\Form\GuideForm.
 */

namespace Drupal\guide\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\guide\Entity\DialogInterface;
use Drupal\guide\Entity\Question;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\SessionManagerInterface;
use Drupal\user\PrivateTempStoreFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Contribute form.
 */
class GuideForm extends FormBase {

  /**
   * @var \Drupal\user\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * @var \Drupal\Core\Session\SessionManagerInterface
   */
  private $sessionManager;

  /**
   * @var \Drupal\Core\Session\AccountInterface
   */
  private $currentUser;

  /**
   * @var \Drupal\user\PrivateTempStore
   */
  protected $store;

  /**
   * Constructs a \Drupal\guide\Form\GuideForm.
   *
   * @param \Drupal\user\PrivateTempStoreFactory $temp_store_factory
   * @param \Drupal\Core\Session\SessionManagerInterface $session_manager
   * @param \Drupal\Core\Session\AccountInterface $current_user
   */
  public function __construct(PrivateTempStoreFactory $temp_store_factory, SessionManagerInterface $session_manager, AccountInterface $current_user) {
    $this->tempStoreFactory = $temp_store_factory;
    $this->sessionManager = $session_manager;
    $this->currentUser = $current_user;

    $this->store = $this->tempStoreFactory->get('guide_data');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('user.private_tempstore'),
      $container->get('session_manager'),
      $container->get('current_user')
    );
  }

  /**
   * Saves the data from the guide form.
   */
  protected function saveData() {
    // Logic for saving data goes here...
    $this->deleteStore();
    //drupal_set_message($this->t('The form has been saved.'));

  }

  /**
   * Helper method that removes all the keys from the store collection used for
   * the guide form.
   */
  protected function deleteStore() {
    $keys = ['delta', 'decisions', 'filter'];
    foreach ($keys as $key) {
      $this->store->delete($key);
    }
  }



  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'guide_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, DialogInterface $dialog = NULL) {

    // Start a manual session for anonymous users.
    if ($this->currentUser->isAnonymous() && !isset($_SESSION['guide_form_session'])) {
      $_SESSION['guide_form_session'] = true;
      $this->sessionManager->start();
      $this->store->set('delta', 0);
    }

    // Set GuideForm title and introduction text.
    $form['#title'] = $dialog->getName();
    $form['intro'] = array(
      '#type' => 'markup',
      '#markup' => $dialog->get('text')->getValue()[0]['value'],
    );

    // Load question logic.
    $conditions = [];
    foreach ($dialog->get('conditions') as $condition) {
      $values = explode('.', $condition->value);
      $conditions[$values[0]][$values[1]] = $values[2];
    }

    // Get current decisions out of session store.
    $decisions = $this->store->get('decisions') ? $this->store->get('decisions') : [];

    // Get last answered question id.
    $last_decision = end($decisions);
    $last_question_id = $last_decision["question"];
    reset($decisions);

    $question_ids = $dialog->get('questions')->getValue();
    $question = NULL;
    foreach ($question_ids as $delta => $question_id) {
      $qid = $question_id["target_id"];
      if ($this->questionAnswered($qid, $decisions)) {
        continue;
      }
      else if (!$this->questionTriggered($qid, $conditions, $decisions)) {
        continue;
      }
      else {
        $question = Question::load($qid);
        $answers = $question->get('answers')->getValue();
        $select_options = [];
        foreach ($answers as $answer) {
          $select_options[$answer['value']] = $answer['text'];
        }

        $form['question'] = [
          '#type' => 'hidden',
          '#value' => $question->id(),
        ];
        $form['answer'] = [
          '#type' => 'radios',
          '#title' => strip_tags($question->get('text')->getValue()[0]['value']),
          '#options' => $select_options,
          '#required' => TRUE,
        ];
        break;
      }
    }

    $form['actions'] = [
      '#type' => 'actions',
      '#weight' => 9,
    ];

    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#name' => 'next',
      '#value' => $this->t('Next'),
      '#button_type' => 'primary',
      '#weight' => 10,
    );

    if (is_null($question)) {
      $form['nomore'] = array(
        '#type' => 'markup',
        '#markup' => "<div>" . t('There are no more questions left. Click submit to see suggested results for your inquiry.') . "</div>",
      );

      $form['actions']['submit']['#value'] = $this->t('Submit');
      $form['actions']['submit']['#name'] = 'submit';
    }

    $form['debug'] = array(
      '#type' => 'markup',
      '#markup' => "<pre>Decisions: " . print_r(($this->store->get('decisions') ? $this->store->get('decisions') : []), 1) . "\nConditions: " . print_r($conditions, 1) . "\nFilter: " . print_r(($this->store->get('filter') ? $this->store->get('filter') : []), 1) . "</pre>",
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    $decisions = $this->store->get('decisions') ? $this->store->get('decisions') : [];
    $decisions[] = [
      "question" => $values["question"],
      "answer" => $values["answer"],
    ];
    $this->store->set('decisions', $decisions);

    $dialog = \Drupal::request()->attributes->get('dialog');
    $target = $dialog->get('target')->getValue()[0]['value'];

    /** @var \Drupal\facets\FacetManager\DefaultFacetManager $facet_manager */
    $facet_manager = \Drupal::service('facets.manager');

    $enabled_facets = $facet_manager->getEnabledFacets();
    $allowed_facets = [];
    foreach ($enabled_facets as $facet) {
      $facet_source_id = $facet->getFacetSourceId();
      \Drupal::logger('guide')->notice($facet_source_id . " <> " . $target);
      if ($facet_source_id == $target) {
        $allowed_facets[] = $facet;
      }
    }

    // Get current filter settings from previous answers.
    $filter = $this->store->get("filter") ? $this->store->get("filter") : [];
    // No redirect to node (yet).
    $redirect_nid = FALSE;

    // Check if a new answer was given.
    if (array_key_exists("question", $values)) {
      $question = Question::load($values["question"]);
      $answers = $question->get("answers")->getValue();
      foreach ($answers as $answer) {
        if ($answer["value"] == $values["answer"]) {
          $options = $answer["options"];
          // Before processing possible filters, check if a given answer should
          // redirect to a specific node.
          if (array_key_exists("redirect", $options)) {
            $redirect_nid = $options["redirect"];
          }
          // No redirect, let's process the filters.
          else if (array_key_exists("facets", $options)) {
            foreach ($options["facets"] as $facetid) {
              foreach ($allowed_facets as $facet) {
                if ($facet->id() == $facetid) {
                  $alias = $facet->getUrlAlias();
                  $filter_values = $options["values"]["facet-" . $facetid];
                  foreach ($filter_values as $filter_value) {
                    $filter[] = $alias . ":" . $filter_value;
                  }
                }
              }
            }
          }
        }
      }
    }
    $this->store->set('filter', $filter);

    $button_clicked = $form_state->getTriggeringElement()['#name'];
    if ($redirect_nid) {
      // Redirect detected. Delete form session and redirect to node.
      $url = Url::fromRoute('entity.node.canonical')
        ->setRouteParameters(array('node' => $redirect_nid));
      $this->deleteStore();
      $form_state->setRedirectUrl($url);
    }
    else if ($button_clicked == 'submit') {
      // Final question reached. Redirect to target facet source.
      $facet = $allowed_facets[0];
      $source = $facet->getFacetSource();

      $query = [];
      foreach ($filter as $key => $val) {
        $query['f[' . $key . ']'] = $val;
      }
      $url = Url::fromUserInput($source->getPath(), [
        'query' => $query,
      ]);
      $this->deleteStore();
      $form_state->setRedirectUrl($url);
    }
    else if ($button_clicked == 'next') {
      // Filters saved, go on with next question.
      $dialog = \Drupal::request()->attributes->get('dialog');
      $url = Url::fromRoute('guide.form')
        ->setRouteParameters(array('dialog' => $dialog->id()));
      $form_state->setRedirectUrl($url);
    }
  }

  /**
   * Decide if a question has been answered.
   *
   * @param $question_id
   * @param $decisions
   * @return bool
   */
  public function questionAnswered($question_id, $decisions) {
    foreach ($decisions as $decision) {
      if ($decision["question"] == $question_id) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Decide if a question has either no condition set
   * or at least one condition is met.
   *
   * @param $question_id
   * @param $conditions
   * @param $decisions
   * @return bool
   */
  public function questionTriggered($question_id, $conditions, $decisions) {
    if (array_key_exists($question_id, $conditions)) {
      foreach ($conditions[$question_id] as $source_question => $source_answer) {
        foreach ($decisions as $decision) {
          if ($decision["question"] == $source_question
          && $decision["answer"] == $source_answer) {
            return TRUE;
          }
        }
      }
      return FALSE;
    }
    else {
      return TRUE;
    }
  }
}
